---
apiVersion: v1
kind: ConfigMap
metadata:
  name: pystol-config
  namespace: pystol
data:
  PYSTOL_UI_FLASK_APP: "{{ appSettings.pystol.ui.app }}"
  PYSTOL_UI_PORT: "{{ appSettings.pystol.ui.port }}"
  PYSTOL_UI_HOST: "{{ appSettings.pystol.ui.host }}"
