---
apiVersion: apps/v1
kind: Deployment
metadata:
  name: pystol-controller
  namespace: pystol
spec:
  selector:
    matchLabels:
      name: pystol-controller
      component: pystol-controller
  replicas: 1
  template:
    metadata:
      labels:
        name: pystol-controller
        component: pystol-controller
    spec:
      containers:
        - name: pystol-controller
          image: {{ appSettings.pystol.controller.image }}
          imagePullPolicy: Always
          command: ["pystol"]
          args: ["listen"]
      serviceAccountName: pystol
